/*
 * WebSocket based DelonKun
 * 
 * Copyright (C) 2012 Yuuichi Akagawa
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ammlab.android.websocketdelonkun;

import java.util.Date;
import io.socket.IOAcknowledge;
import io.socket.IOCallback;
import io.socket.SocketIO;
import io.socket.SocketIOException;

import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.res.Resources;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;


public class WebSocketDelonkunActivity extends Activity {
	private static final String TAG = "WebSocketDelonkun";
	private static final String sURI = "http://WWW.EXAMPLE.COM:PORT/in";
	private static final int MESSAGE_CONNECT    = 1;
	private static final int MESSAGE_DISCONNECT = 2;
	private static final int MESSAGE_UPDATE     = 3;
	   
	private SocketIO socket;
    private TextView mTextView1;
    private TextView mTextView2;
    private Button mButton;
    ToneGenerator toneGenerator;
    int patternDTMF[] ;
    String nameCommand[] ;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        mTextView1 = (TextView) findViewById(R.id.textView1);
        mTextView2 = (TextView) findViewById(R.id.textView2);
        mButton    = (Button)   findViewById(R.id.button1);
        
        //DTMF
        toneGenerator = new
        ToneGenerator(AudioManager.STREAM_SYSTEM,ToneGenerator.MAX_VOLUME);

        //Command to DTMF conversion table
        patternDTMF = new int[10];
        patternDTMF[1] = ToneGenerator.TONE_DTMF_4;
        patternDTMF[2] = ToneGenerator.TONE_DTMF_5;
        patternDTMF[3] = ToneGenerator.TONE_DTMF_1;
        patternDTMF[4] = ToneGenerator.TONE_DTMF_6;
        patternDTMF[5] = ToneGenerator.TONE_DTMF_D;
        patternDTMF[6] = ToneGenerator.TONE_DTMF_9;
        patternDTMF[7] = ToneGenerator.TONE_DTMF_8;
        patternDTMF[8] = ToneGenerator.TONE_DTMF_0;
        patternDTMF[9] = ToneGenerator.TONE_DTMF_2;
        
        //Command No to Command Name
        nameCommand = new String[10];
        nameCommand[1] = new String("Forward Left");
        nameCommand[2] = new String("Forward");
        nameCommand[3] = new String("Forward Rigiht");
        nameCommand[4] = new String("Turn Left");
        nameCommand[5] = new String("Stop");
        nameCommand[6] = new String("Turn Right");
        nameCommand[7] = new String("Back Left");
        nameCommand[8] = new String("Back");
        nameCommand[9] = new String("Back Rigiht");

        //Socket.IO
		socket     = new SocketIO();
		startConnection();
		
		//~{^
		mButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				toneGenerator.startTone(patternDTMF[5]);	
            }
        });
   }

    @Override
	public void onResume() {
		Log.v(TAG, "onResume()");
		super.onResume();
		startConnection();
	}
	
    @Override
    public void onPause(){
		Log.v(TAG, "onPause()");
		super.onPause();
    }
    @Override
    public void onStop(){
		Log.v(TAG, "onStop()");
		toneGenerator.stopTone();
		super.onStop();
    }
    @Override
    public void onDestroy(){
		Log.v(TAG, "onDestroy()");
    	socket.disconnect();
    	super.onDestroy();
    }
    
    // ڑ
    private void startConnection(){
    	// ڑς݁H
    	if( socket.isConnected() == true){
    		socket.reconnect();
    		return;
    	}
    	
		try {
			socket.connect(sURI, new IOCallback() {
				@Override
				public void onMessage(JSONObject json, IOAcknowledge ack) {
					try {
						Log.v(TAG,"Server said:" + json.toString(2));
					} catch (JSONException e) {
						e.printStackTrace();
					}
				}

				@Override
				public void onMessage(String data, IOAcknowledge ack) {
					Log.v(TAG,"Server said: " + data);
				}

				@Override
				public void onError(SocketIOException socketIOException) {
					Log.v(TAG,"an Error occured");
					socketIOException.printStackTrace();
				}

				@Override
				public void onConnect() {
					Log.v(TAG,"Connection established");
                    Message m = Message.obtain(mHandler, MESSAGE_CONNECT);
                    mHandler.sendMessage(m);
				}

				@Override
				public void onDisconnect() {
					Log.v(TAG,"Connection terminated.");
                    Message m = Message.obtain(mHandler, MESSAGE_DISCONNECT);
                    mHandler.sendMessage(m);
                }

				@Override
				public void on(String event, IOAcknowledge ack, Object... args) {
					Log.v(TAG,"Server triggered event '" + event);
                    Message m = Message.obtain(mHandler, MESSAGE_UPDATE);
                    m.obj = args[0];
                    mHandler.sendMessage(m);
				}					
			});
		}
		catch (Exception e) {
			e.printStackTrace();
		}
    	
    }

    // UI Xbhŉʏ̕\ύX
    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MESSAGE_CONNECT:
                	updateTitle(true);
                	break;
                case MESSAGE_DISCONNECT:
                	updateTitle(false);
                	break;
                case MESSAGE_UPDATE:
                	JSONObject  json = (JSONObject) msg.obj;
					try {
						int cmd = Integer.parseInt(json.getString("cmd"));
						Date dt = new Date(Long.valueOf(json.getString("time")));
						mTextView1.setText(dt.toString());
						mTextView2.setText("Command: " + nameCommand[cmd] +" (" + cmd + ")");
						//Play DTMF
						playDTMF(patternDTMF[cmd]);
					} catch (JSONException e) {
						e.printStackTrace();
					}
                    break;
            }
        }
    };
 
    // ڑԕ\(^Cgo[)
    private void updateTitle(boolean enable) {
    	Resources res = getResources();
    	String titleString = res.getString(R.string.app_name);
 
        if (enable) {
            setTitle(titleString + " - connected");
        } else {
            setTitle(titleString + " - not connected");
        }
    }
    
    // DTMFMĐ
    private void playDTMF(int tone){
    	toneGenerator.startTone(tone, 3000);   // Duration = 3000ms
    }
}